@@ -3,17 +3,21 @@ use warnings;
 
 package Mojolicious::Plugin::DbicSchemaViewer;
 
-our $VERSION = '0.0101'; # VERSION:
 # ABSTRACT: Viewer for DBIx::Class schema definitions
+our $AUTHORITY = 'cpan:CSSON'; # AUTHORITY
+our $VERSION = '0.0102';
 
 use Mojo::Base 'Mojolicious::Plugin';
 use File::ShareDir::Tarball 'dist_dir';
 use Path::Tiny;
 use Data::Dump::Streamer;
 use Safe::Isa;
+use DateTime::Tiny;
+
 use experimental qw/signatures postderef/;
 
 sub register($self, $app, $conf) {
+    $app->plugin('BootstrapHelpers');
 
     # check configuration
     if(exists $conf->{'router'} && exists $conf->{'condition'}) {
@@ -44,9 +48,17 @@ sub register($self, $app, $conf) {
     my $url = $conf->{'url'} || 'dbic-schema-viewer';
     my $schema = $conf->{'schema'};
 
+    my $can_show_visualizer = eval { require DBIx::Class::Visualizer; 1; };
+
     $router->get($url)->to(cb => sub ($c) {
         $self->render($c, 'viewer/schema', db => $self->schema_info($schema), schema_name => ref $schema);
-    });
+    })->name('schema');
+    $router->get("$url/visualizer")->to(cb => sub ($c) {
+        $self->render($c, 'viewer/visualizer', schema_name => ref $schema, can_show_visualizer => $can_show_visualizer);
+    })->name('visualizer');
+    $router->get("$url/visualizer/svg")->to(cb => sub ($c) {
+        $c->render(data => $self->visualizer($schema));
+    })->name('visualizer_svg');
 }
 
 sub render($self, $c, $template, @args) {
@@ -102,20 +114,29 @@ sub schema_info($self, $schema) {
 
         foreach my $relation_name (sort $rs->relationships) {
             my $relation = $rs->relationship_info($relation_name);
-
             my $class_name = $relation->{'class'} =~ s{^.*?::Result::}{}r;
 
-            my $condition = Dump($relation->{'cond'})->Out;
+            my $condition;
+            # simple one column to one column relation: this_result_id => relation_name.that_result_id
+            if(ref $relation->{'cond'} eq 'HASH' && scalar keys $relation->{'cond'}->%* == 1) {
+                my @cleaned_condition = ((values $relation->{'cond'}->%*)[0] =~ s{^self\.}{}rx);
+                push @cleaned_condition => (keys $relation->{'cond'}->%*)[0] =~ s{^foreign(?=\.)}{$relation_name}rx;
+                $condition = join ' => ', @cleaned_condition;
+            }
+            # more complicated relation: dump relation to text and remove boilerplate
+            else {
+                $condition = Dump($relation->{'cond'})->Out;
 
-            # cleanup the dump
-            $condition =~ s{^.*?\{}{\{};
-            $condition =~ s{\n\s*?package .*?\n}{\n};
-            $condition =~ s{\n\s*?BEGIN.*?\n}{\n};
-            $condition =~ s{\n\s*?use strict.*\n}{\n}g;
-            $condition =~ s{\n\s*?use feature.*\n}{\n}g;
-            $condition =~ s{\n\s*?no feature.*\n}{\n}g;
-            $condition =~ s{\n\s{3,}\}}{\n\}};
-            $condition =~ s{\n\s{8,8}}{\n    }g;
+                # cleanup the dump
+                $condition =~ s{^.*?\{}{\{};
+                $condition =~ s{\n\s*?package .*?\n}{\n};
+                $condition =~ s{\n\s*?BEGIN.*?\n}{\n};
+                $condition =~ s{\n\s*?use strict.*?\n}{\n}g;
+                $condition =~ s{\n\s*?use feature.*?\n}{\n}g;
+                $condition =~ s{\n\s*?no feature.*?\n}{\n}g;
+                $condition =~ s{\n\s{3,}\}}{\n\}};
+                $condition =~ s{\n\s{8,8}}{\n    }g;
+            }
 
             my $on_cascade = [ sort map { $_ =~ s{^cascade_}{}rm } grep { m/^cascade/ && $relation->{'attrs'}{ $_ } } keys $relation->{'attrs'}->%* ];
 
@@ -135,6 +156,10 @@ sub schema_info($self, $schema) {
     return $db;
 }
 
+sub visualizer($self, $schema) {
+    return DBIx::Class::Visualizer->new(schema => $schema)->svg;
+}
+
 1;
 
 __END__
@@ -149,22 +174,21 @@ Mojolicious::Plugin::DbicSchemaViewer - Viewer for DBIx::Class schema definition
 
 
 
-=begin HTML
+=begin html
 
-<p><img src="https://img.shields.io/badge/perl-5.20-brightgreen.svg" alt="Requires Perl 5.20" /> <a href="https://travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer"><img src="https://api.travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer.svg?branch=master" alt="Travis status" /></a></p>
+<p>
+<img src="https://img.shields.io/badge/perl-5.20+-blue.svg" alt="Requires Perl 5.20+" />
+<a href="https://travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer"><img src="https://api.travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer.svg?branch=master" alt="Travis status" /></a>
+<a href="http://cpants.cpanauthors.org/release/CSSON/Mojolicious-Plugin-DbicSchemaViewer-0.0102"><img src="http://badgedepot.code301.com/badge/kwalitee/CSSON/Mojolicious-Plugin-DbicSchemaViewer/0.0102" alt="Distribution kwalitee" /></a>
+<a href="http://matrix.cpantesters.org/?dist=Mojolicious-Plugin-DbicSchemaViewer%200.0102"><img src="http://badgedepot.code301.com/badge/cpantesters/Mojolicious-Plugin-DbicSchemaViewer/0.0102" alt="CPAN Testers result" /></a>
+<img src="https://img.shields.io/badge/coverage-65.3%-red.svg" alt="coverage 65.3%" />
+</p>
 
-=end HTML
-
-
-=begin markdown
-
-![Requires Perl 5.20](https://img.shields.io/badge/perl-5.20-brightgreen.svg) [![Travis status](https://api.travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer.svg?branch=master)](https://travis-ci.org/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer)
-
-=end markdown
+=end html
 
 =head1 VERSION
 
-Version 0.0101, released 2015-10-28.
+Version 0.0102, released 2016-09-04.
 
 =head1 SYNOPSIS
 
@@ -174,7 +198,10 @@ Version 0.0101, released 2015-10-28.
 
 =head1 DESCRIPTION
 
-This plugin is viewer for L<DBIx::Class> schemas. It lists all C<ResultSources> with column definitions and and their relationships. See C<examples/example.html> for an example.
+This plugin is a viewer for L<DBIx::Class> schemata. It lists all C<ResultSources> with column definitions and and their relationships. See C<examples/example.html> for
+an example (also available on L<Github|http://htmlpreview.github.io/?https://github.com/Csson/p5-Mojolicious-Plugin-DbicSchemaViewer/blob/master/examples/example.html>).
+
+Optionally, if L<DBIx::Class::Visualizer> is installed, a graphical representation of the schema can be rendered using L<GraphViz2>.
 
 =head2 Configuration
 
@@ -184,7 +211,7 @@ The following settings are available. It is recommended to use either L</router>
 
 Mandatory.
 
-Should be an instance of an C<DBIx::Class::Schema> class.
+An instance of a C<DBIx::Class::Schema> class.
 
 =head3 url
 
@@ -197,7 +224,7 @@ By default, the viewer is located at C</dbic-schema-viewer>.
         schema => Your::Schema->connect(...),
     });
 
-Now the viewer is located at C</the-schema>.
+The viewer is instead located at C</the-schema>.
 
 =head3 router
 
@@ -244,7 +271,7 @@ Erik Carlsson <info@code301.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Erik Carlsson <info@code301.com>.
+This software is copyright (c) 2016 by Erik Carlsson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
